/// @file       glcd12864lib.c
/// @copyright  Copyright (c) 2014 Toradex AG
/// @author     $Author: ras$
/// @date       $Date: 2014-03-24 12:00:00 +05:30$
/// @brief      This library allows easy access to the Graphical LCD with Toradex module.
///

#include <windows.h>
#include "gpioLib.h"
#include "glcd12864lib.h"
#include "tahoma8x8.h"

BYTE glcdDataPin[] = {GLCD_DATA0, GLCD_DATA1, GLCD_DATA2, GLCD_DATA3, GLCD_DATA4, GLCD_DATA5, GLCD_DATA6, GLCD_DATA7};

//******************************************************************************
void GlcdConfig(void)
{
    InitGPIOLib();                           ///< Initialize GPIO library
    /// Configure GPIOs as output
    SetPinAltFn(GLCD_EN, -1, DIR_OUT);
    SetPinAltFn(GLCD_RW, -1, DIR_OUT);
    SetPinAltFn(GLCD_RS, -1, DIR_OUT);
    SetPinAltFn(GLCD_RST, -1, DIR_OUT);
    SetPinAltFn(GLCD_CS, -1, DIR_OUT);
    SetPinAltFn(GLCD_C86, -1, DIR_OUT);
    SetPinAltFn(GLCD_DATA0, -1, DIR_OUT);
    SetPinAltFn(GLCD_DATA1, -1, DIR_OUT);
    SetPinAltFn(GLCD_DATA2, -1, DIR_OUT);
    SetPinAltFn(GLCD_DATA3, -1, DIR_OUT);
    SetPinAltFn(GLCD_DATA4, -1, DIR_OUT);
    SetPinAltFn(GLCD_DATA5, -1, DIR_OUT);
    SetPinAltFn(GLCD_DATA6, -1, DIR_OUT);
    SetPinAltFn(GLCD_DATA7, -1, DIR_OUT);
    SetPinAltFn(GLCD_LEDA, -1, DIR_OUT);

    /// Set default GPIO pin Level as OFF
    SetPinLevel(GLCD_EN, FALSE);
    SetPinLevel(GLCD_RW, FALSE);
    SetPinLevel(GLCD_RS, FALSE);
    SetPinLevel(GLCD_RST, FALSE);
    SetPinLevel(GLCD_CS, FALSE);
    SetPinLevel(GLCD_C86, FALSE);
    SetPinLevel(GLCD_DATA0, FALSE);
    SetPinLevel(GLCD_DATA1, FALSE);
    SetPinLevel(GLCD_DATA2, FALSE);
    SetPinLevel(GLCD_DATA3, FALSE);
    SetPinLevel(GLCD_DATA4, FALSE);
    SetPinLevel(GLCD_DATA5, FALSE);
    SetPinLevel(GLCD_DATA6, FALSE);
    SetPinLevel(GLCD_DATA7, FALSE);
    SetPinLevel(GLCD_LEDA, FALSE);
}

//******************************************************************************
void GlcdDeInit(void)
{
    GlcdBackLight(FALSE);
    GlcdClear();
    DeInitGPIOLib();        
}

//******************************************************************************
void GlcdInit(void)
{
    GlcdConfig(); 
    Sleep(1);
    SetPinLevel(GLCD_C86, FALSE);       ///< Select Parallel Data Mode
    SetPinLevel(GLCD_EN, TRUE);
    Sleep(10);
    SetPinLevel(GLCD_RST, FALSE);       ///< Clear Reset Pin
    Sleep(1);     
    SetPinLevel(GLCD_RST, TRUE);        ///< Set Reset Pin
    GlcdWriteCmd(0xE2);                 ///< Software Reset GLCD
    GlcdDelay(USDELAY);  

    /// Send Initialization commands to GLCD
    GlcdWriteCmd(0xA0);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0xC8);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0xA3);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0x2F);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0x21);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0x81);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0x40);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0xA4);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0xA6);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0x81);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0x3F);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0xAF);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0xB0);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0x10);
    GlcdDelay(USDELAY);
    GlcdWriteCmd(0x00);
    GlcdDelay(USDELAY);
}

//******************************************************************************
void GlcdClear(void)
{
    BYTE glcdPage = 0;
    BYTE glcdColumn = 0;

    GlcdSetPage(GLCD_PAGE1, 0);                     ///< Initial page and column

    for (glcdPage = 0; glcdPage < 8; glcdPage++)
    {
        for (glcdColumn = 0; glcdColumn < 128; glcdColumn++)
        {
            GlcdWriteData(0x00);
        }
        GlcdSetPage((GLCD_PAGE1+glcdPage+1), 0);    ///< Increment page
    }
    GlcdSetPage(GLCD_PAGE1, 0);                     ///< Initial page and column
}

//******************************************************************************
void GlcdSetPage(BYTE glcdPageNum,BYTE glcdColumnNum)
{
    BYTE glcdMsb = 0;
    BYTE glcdLsb = 0;
    glcdMsb = (((glcdColumnNum & 0xF0) >> 4) | 0x10);
    glcdLsb = (glcdColumnNum & 0x0F);
    GlcdWriteCmd(glcdPageNum);
    GlcdWriteCmd(glcdMsb);
    GlcdWriteCmd(glcdLsb);
}

//******************************************************************************
void GlcdWriteData(BYTE glcdData)
{
    SetPinLevel(GLCD_RS, TRUE);
    GlcdDelay(USDELAY);
    SetPinLevel(GLCD_CS, FALSE);
    GlcdDelay(USDELAY);
    SetPinLevel(GLCD_RW, FALSE);    
    GlcdDelay(USDELAY);

    /// Send data to GLCD
    GlcdSendDataByte(glcdData); 
    GlcdDelay(USDELAY); 

    SetPinLevel(GLCD_RW, TRUE);
    GlcdDelay(USDELAY);
    SetPinLevel(GLCD_CS, TRUE);
    GlcdDelay(USDELAY);
    SetPinLevel(GLCD_RS, FALSE);
    GlcdDelay(USDELAY);
}

//******************************************************************************
void GlcdWriteCmd(BYTE glcdCmd)
{
    SetPinLevel(GLCD_RS, FALSE);
    GlcdDelay(USDELAY);
    SetPinLevel(GLCD_CS, FALSE);
    GlcdDelay(USDELAY);
    SetPinLevel(GLCD_RW, FALSE);
    GlcdDelay(USDELAY);

    /// Send command to GLCD
    GlcdSendDataByte(glcdCmd);
    GlcdDelay(USDELAY);

    SetPinLevel(GLCD_RW, TRUE);
    GlcdDelay(USDELAY);
    SetPinLevel(GLCD_CS, TRUE);
    GlcdDelay(USDELAY);
    SetPinLevel(GLCD_RS, TRUE);
    GlcdDelay(USDELAY);
}

//******************************************************************************
void GlcdSendDataByte(BYTE glcdDataByte)
{
    BYTE glcdBitCounter = 0;
    BYTE glcdDataBit = 0;
    BYTE glcdDataSave = glcdDataByte;

    for (glcdBitCounter = 0; glcdBitCounter < 8; glcdBitCounter++)
    {
        glcdDataByte = glcdDataSave;
        glcdDataBit = ((glcdDataByte>>glcdBitCounter) & 0x01);
        SetPinLevel(glcdDataPin[glcdBitCounter], glcdDataBit);
    }
}

//******************************************************************************
void GlcdBackLight(BYTE glcdLedStatus)
{
    SetPinLevel(GLCD_LEDA, glcdLedStatus);
}

//******************************************************************************
void GlcdDelay(DWORD usDelay)
{
    DWORD usDelayCount;
    do{
        usDelayCount = 10;
        while(--usDelayCount);
    }while(--usDelay);
}

//******************************************************************************
void GlcdDisplayChar(BYTE character)
{
  BYTE fontWidth = 0;
  BYTE fontIndex = 0;
  BYTE fontData = 0;

  fontWidth = getWidthOfChar(character);
  for (fontIndex = 0; fontIndex < fontWidth; fontIndex++)
  {
    fontData = getDataFromDataTable(character, fontIndex);
    GlcdWriteData(fontData);
  }
}
//******************************************************************************
void GlcdDisplayString(BYTE *string)
{
  if (string == NULL) return;

  while (*string != '\0')
  {
      GlcdDisplayChar(*string);
      string++;
  }
}

